@@ -96,7 +96,7 @@ typedef enum {
 
 #define MAX(a, b) (((a)>(b))?(a):(b))
 #define MIN(a, b) (((a)<(b))?(a):(b))
-#define TIME_DIFF(a, b) ((long)((a)-(b)))
+#define TIME_DIFF(a, b) ((int)((a)-(b)))
 #define SYSCALL(call) while (((call) == -1) && (errno == EINTR))
 
 #ifndef M_PI
@@ -276,7 +276,6 @@ SynapticsPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     pointer optList;
 #endif
     char *str_par;
-    unsigned long now;
     SynapticsSHM *pars;
     char *repeater;
 
@@ -311,7 +310,7 @@ SynapticsPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     local->history_size            = 0;
     local->always_core_feedback    = 0;
 
-    xf86Msg(X_INFO, "Synaptics touchpad driver version %s\n", VERSION);
+    xf86Msg(X_INFO, "Synaptics touchpad driver version %s (%d)\n", VERSION, VERSION_ID);
 
     xf86CollectInputOptions(local, NULL, NULL);
 
@@ -331,18 +330,18 @@ SynapticsPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     priv->timer = NULL;
     priv->repeatButtons = 0;
     priv->nextRepeat = 0;
-    now = GetTimeInMillis();
     priv->count_packet_finger = 0;
     priv->tap_state = TS_START;
     priv->tap_button = 0;
     priv->tap_button_state = TBS_BUTTON_UP;
-    priv->touch_on.millis = now;
+    priv->touch_on.millis = 0;
 
     /* install shared memory or normal memory for parameters */
     priv->shm_config = xf86SetBoolOption(local->options, "SHMConfig", FALSE);
 
     /* read the parameters */
     pars = &priv->synpara_default;
+    pars->version = VERSION_ID;
     pars->left_edge = xf86SetIntOption(local->options, "LeftEdge", 1900);
     pars->right_edge = xf86SetIntOption(local->options, "RightEdge", 5400);
     pars->top_edge = xf86SetIntOption(local->options, "TopEdge", 1900);
@@ -365,6 +364,10 @@ SynapticsPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
     pars->edge_motion_use_always = xf86SetBoolOption(local->options, "EdgeMotionUseAlways", FALSE);
     repeater = xf86SetStrOption(local->options, "Repeater", NULL);
     pars->updown_button_scrolling = xf86SetBoolOption(local->options, "UpDownScrolling", TRUE);
+    pars->leftright_button_scrolling = xf86SetBoolOption(local->options, "LeftRightScrolling", TRUE);
+    pars->updown_button_repeat = xf86SetBoolOption(local->options, "UpDownScrollRepeat", TRUE);
+    pars->leftright_button_repeat = xf86SetBoolOption(local->options, "LeftRightScrollRepeat", TRUE);
+    pars->scroll_button_repeat = xf86SetIntOption(local->options,"ScrollButtonRepeat", 100);
     pars->touchpad_off = xf86SetIntOption(local->options, "TouchpadOff", 0);
     pars->guestmouse_off = xf86SetBoolOption(local->options, "GuestMouseOff", FALSE);
     pars->locked_drags = xf86SetBoolOption(local->options, "LockedDrags", FALSE);
@@ -522,6 +525,7 @@ DeviceOn(DeviceIntPtr dev)
 
     DBG(3, ErrorF("Synaptics DeviceOn called\n"));
 
+    SetDeviceAndProtocol(local);
     local->fd = xf86OpenSerial(local->options);
     if (local->fd == -1) {
 	xf86Msg(X_WARNING, "%s: cannot open input device\n", local->name);
@@ -802,11 +806,11 @@ ReadInput(LocalDevicePtr local)
 }
 
 static int
-HandleMidButtonEmulation(SynapticsPrivate *priv, struct SynapticsHwState *hw, long *delay)
+HandleMidButtonEmulation(SynapticsPrivate *priv, struct SynapticsHwState *hw, int *delay)
 {
     SynapticsSHM *para = priv->synpara;
     Bool done = FALSE;
-    long timeleft;
+    int timeleft;
     int mid = 0;
 
     while (!done) {
@@ -930,7 +934,7 @@ SelectTapButton(SynapticsPrivate *priv, edge_type edge)
     if (priv->synpara->touchpad_off == 2) {
 	priv->tap_button = 0;
 	return;
-    }    
+    }
 
     switch (priv->tap_max_fingers) {
     case 1:
@@ -988,7 +992,7 @@ SetTapState(SynapticsPrivate *priv, enum TapState tap_state, int millis)
     case TS_2A:
 	if (para->fast_taps)
 	    priv->tap_button_state = TBS_BUTTON_DOWN;
-	else 
+	else
 	    priv->tap_button_state = TBS_BUTTON_UP;
 	break;
     case TS_2B:
@@ -1000,7 +1004,7 @@ SetTapState(SynapticsPrivate *priv, enum TapState tap_state, int millis)
     case TS_SINGLETAP:
 	if (para->fast_taps)
 	    priv->tap_button_state = TBS_BUTTON_UP;
-	else 
+	else
 	    priv->tap_button_state = TBS_BUTTON_DOWN;
 	priv->touch_on.millis = millis;
 	break;
@@ -1036,8 +1040,8 @@ HandleTapProcessing(SynapticsPrivate *priv, struct SynapticsHwState *hw,
 {
     SynapticsSHM *para = priv->synpara;
     Bool touch, release, is_timeout, move;
-    long timeleft, timeout;
-    long delay = 1000000000;
+    int timeleft, timeout;
+    int delay = 1000000000;
 
     if (priv->palm)
 	return delay;
@@ -1160,7 +1164,7 @@ estimate_delta(double x0, double x1, double x2, double x3)
     return x0 * 0.3 + x1 * 0.1 - x2 * 0.1 - x3 * 0.3;
 }
 
-static long
+static int
 ComputeDeltas(SynapticsPrivate *priv, struct SynapticsHwState *hw,
 	      edge_type edge, int *dxP, int *dyP)
 {
@@ -1169,7 +1173,7 @@ ComputeDeltas(SynapticsPrivate *priv, struct SynapticsHwState *hw,
     int dist;
     double dx, dy;
     double speed, integral;
-    long delay = 1000000000;
+    int delay = 1000000000;
 
     dx = dy = 0;
 
@@ -1278,7 +1282,8 @@ struct ScrollData {
 };
 
 static void
-start_coasting(SynapticsPrivate *priv, struct SynapticsHwState *hw, edge_type edge)
+start_coasting(SynapticsPrivate *priv, struct SynapticsHwState *hw, edge_type edge,
+	       Bool vertical)
 {
     SynapticsSHM *para = priv->synpara;
 
@@ -1287,8 +1292,7 @@ start_coasting(SynapticsPrivate *priv, struct SynapticsHwState *hw, edge_type ed
 
     if ((priv->scroll_packet_count > 3) && (para->coasting_speed > 0.0)) {
 	double pkt_time = (HIST(0).millis - HIST(3).millis) / 1000.0;
-	if (priv->vert_scroll_on ||
-	    (priv->circ_scroll_on && priv->circ_scroll_vert)) {
+	if (vertical) {
 	    double dy = estimate_delta(HIST(0).y, HIST(1).y, HIST(2).y, HIST(3).y);
 	    int sdelta = para->scroll_dist_vert;
 	    if ((edge & RIGHT_EDGE) && pkt_time > 0 && sdelta > 0) {
@@ -1318,11 +1322,19 @@ HandleScrolling(SynapticsPrivate *priv, struct SynapticsHwState *hw,
 		edge_type edge, Bool finger, struct ScrollData *sd)
 {
     SynapticsSHM *para = priv->synpara;
-    Bool scroll_stop = FALSE;
     int delay = 1000000000;
 
     sd->left = sd->right = sd->up = sd->down = 0;
 
+    if (priv->synpara->touchpad_off == 2) {
+	priv->autoscroll_xspd = 0;
+	priv->autoscroll_yspd = 0;
+	priv->circ_scroll_on = FALSE;
+	priv->vert_scroll_on = FALSE;
+	priv->horiz_scroll_on = FALSE;
+	return delay;
+    }
+
     /* scroll detection */
     if (finger && !priv->finger_flag) {
 	priv->autoscroll_xspd = 0;
@@ -1357,24 +1369,26 @@ HandleScrolling(SynapticsPrivate *priv, struct SynapticsHwState *hw,
 	    }
 	}
     }
-    if (priv->circ_scroll_on && !finger) {
-	/* circular scroll locks in until finger is raised */
-	DBG(7, ErrorF("cicular scroll off\n"));
-	scroll_stop = TRUE;
-    }
-    if (priv->vert_scroll_on && (!(edge & RIGHT_EDGE) || !finger)) {
-	DBG(7, ErrorF("vert edge scroll off\n"));
-	scroll_stop = TRUE;
-    }
-    if (priv->horiz_scroll_on && (!(edge & BOTTOM_EDGE) || !finger)) {
-	DBG(7, ErrorF("horiz edge scroll off\n"));
-	scroll_stop = TRUE;
-    }
-    if (scroll_stop) {
-	start_coasting(priv, hw, edge);
-	priv->circ_scroll_on = FALSE;
-	priv->vert_scroll_on = FALSE;
-	priv->horiz_scroll_on = FALSE;
+    {
+	Bool oldv = priv->vert_scroll_on || (priv->circ_scroll_on && priv->circ_scroll_vert);
+	Bool oldh = priv->horiz_scroll_on || (priv->circ_scroll_on && !priv->circ_scroll_vert);
+	if (priv->circ_scroll_on && !finger) {
+	    /* circular scroll locks in until finger is raised */
+	    DBG(7, ErrorF("cicular scroll off\n"));
+	    priv->circ_scroll_on = FALSE;
+	}
+	if (priv->vert_scroll_on && (!(edge & RIGHT_EDGE) || !finger)) {
+	    DBG(7, ErrorF("vert edge scroll off\n"));
+	    priv->vert_scroll_on = FALSE;
+	}
+	if (priv->horiz_scroll_on && (!(edge & BOTTOM_EDGE) || !finger)) {
+	    DBG(7, ErrorF("horiz edge scroll off\n"));
+	    priv->horiz_scroll_on = FALSE;
+	}
+	if ((oldv || oldh) && !(priv->circ_scroll_on || priv->vert_scroll_on ||
+			  priv->horiz_scroll_on)) {
+	    start_coasting(priv, hw, edge, oldv);
+	}
     }
 
     /* if hitting a corner (top right or bottom right) while vertical scrolling
@@ -1497,13 +1511,13 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState *hw)
     SynapticsPrivate *priv = (SynapticsPrivate *) (local->private);
     SynapticsSHM *para = priv->synpara;
     Bool finger;
-    int dx, dy, buttons, id;
+    int dx, dy, buttons, rep_buttons, id;
     edge_type edge;
     int change;
     struct ScrollData scroll;
-    int double_click;
-    long delay = 1000000000;
-    long timeleft;
+    int double_click, repeat_delay;
+    int delay = 1000000000;
+    int timeleft;
     int i;
 
     /* update hardware state in shared memory */
@@ -1559,6 +1573,15 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState *hw)
 	hw->up = hw->down = FALSE;
     }
 
+    /* Left/right button scrolling, or middle clicks */
+    if (!para->leftright_button_scrolling) {
+	if (hw->multi[2] || hw->multi[3])
+	    hw->middle = TRUE;
+
+	/* reset left/right button events */
+	hw->multi[2] = hw->multi[3] = FALSE;
+    }
+
     /*
      * Some touchpads have a scroll wheel region where a very large X
      * coordinate is reported. For such touchpads, we adjust the X
@@ -1587,6 +1610,9 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState *hw)
     timeleft = ComputeDeltas(priv, hw, edge, &dx, &dy);
     delay = MIN(delay, timeleft);
 
+    rep_buttons = ((para->updown_button_repeat ? 0x18 : 0) |
+		   (para->leftright_button_repeat ? 0x60 : 0));
+
     buttons = ((hw->left     ? 0x01 : 0) |
 	       (hw->middle   ? 0x02 : 0) |
 	       (hw->right    ? 0x04 : 0) |
@@ -1644,11 +1670,14 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState *hw)
     }
 
     /* Handle auto repeat buttons */
-    if ((hw->up || hw->down || hw->multi[2] || hw->multi[3]) &&
-	para->updown_button_scrolling) {
-	priv->repeatButtons = buttons & 0x78;
+    repeat_delay = clamp(para->scroll_button_repeat, SBR_MIN, SBR_MAX);
+    if (((hw->up || hw->down) && para->updown_button_repeat &&
+	 para->updown_button_scrolling) ||
+	((hw->multi[2] || hw->multi[3]) && para->leftright_button_repeat &&
+	 para->leftright_button_scrolling) ){
+	priv->repeatButtons = buttons & rep_buttons;
 	if (!priv->nextRepeat) {
-	    priv->nextRepeat = hw->millis + 200;
+	    priv->nextRepeat = hw->millis + repeat_delay * 2;
 	}
     } else {
 	priv->repeatButtons = 0;
@@ -1669,8 +1698,8 @@ HandleState(LocalDevicePtr local, struct SynapticsHwState *hw)
 		xf86PostButtonEvent(local->dev, FALSE, id, TRUE, 0, 0);
 	    }
 
-	    priv->nextRepeat = hw->millis + 100;
-	    delay = MIN(delay, 100);
+	    priv->nextRepeat = hw->millis + repeat_delay;
+	    delay = MIN(delay, repeat_delay);
 	}
     }
 
