@@ -72,6 +72,10 @@ static struct Parameter params[] = {
     DEFINE_PAR("EdgeMotionMaxSpeed",   edge_motion_max_speed,   PT_INT,    0, 1000),
     DEFINE_PAR("EdgeMotionUseAlways",  edge_motion_use_always,  PT_BOOL,   0, 1),
     DEFINE_PAR("UpDownScrolling",      updown_button_scrolling, PT_BOOL,   0, 1),
+    DEFINE_PAR("LeftRightScrolling",   leftright_button_scrolling, PT_BOOL,   0, 1),
+    DEFINE_PAR("UpDownRepeat",         updown_button_repeat,    PT_BOOL,   0, 1),
+    DEFINE_PAR("LeftRightRepeat",      leftright_button_repeat, PT_BOOL,   0, 1),
+    DEFINE_PAR("ScrollButtonRepeat",   scroll_button_repeat,    PT_INT,    SBR_MIN , SBR_MAX),
     DEFINE_PAR("TouchpadOff",          touchpad_off,            PT_INT,    0, 2),
     DEFINE_PAR("GuestMouseOff",        guestmouse_off,          PT_BOOL,   0, 1),
     DEFINE_PAR("LockedDrags",          locked_drags,            PT_BOOL,   0, 1),
@@ -106,6 +110,7 @@ show_hw_info(SynapticsSHM *synshm)
 	printf("    This is normal if you are running linux kernel 2.6.\n");
 	printf("    Check the kernel log for touchpad hardware information.\n");
     }
+    printf("Driver version: %d\n", VERSION_ID);
 }
 
 static void
@@ -258,7 +263,7 @@ usage()
 	    "     interval specifies how often (in ms) to poll the touchpad state\n");
     fprintf(stderr, "  -h Show detected hardware properties\n");
     fprintf(stderr, "  -l List current user settings\n");
-    fprintf(stderr, "  -V Print version number and exit\n");
+    fprintf(stderr, "  -V Print synclient version string and exit\n");
     fprintf(stderr, "  -? Show this help message\n");
     fprintf(stderr, "  var=value  Set user parameter 'var' to 'value'.\n");
     exit(1);
